package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import models.Almacenamiento;
import persistance.interfaces.Iabm;

@Path("/Almacenamiento")
@Tag(name = "Almacenamiento", description = "Operaciones relacionadas con almacenamiento")
public class AlmacenamientoResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject	
	private Iabm<Almacenamiento> almaDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todos los almacenamientos",
      description = "Retorna una lista de todos los almacenamientos disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de almacenamientos obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Almacenamiento.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")

    public Response getAlmacenamientos() {
        try {
            List<Almacenamiento> almas = almaDAO.getAll();
            return Response.ok(almas).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban los almacenamiento.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
    @Operation(
        summary = "Obtener almacenamiento por ID",
        description = "Devuelve un almacenamiento específico por su ID"
        )
	@ApiResponse(responseCode = "200", description = "Almacenamiento encontrado",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Almacenamiento.class)))
	@ApiResponse(responseCode = "404", description = "Almacenamiento no encontrado")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getAlmacenamientoById(@PathParam("id") Long id) {
        try {
            Optional<Almacenamiento> alma = almaDAO.get(id);
            if (alma.isPresent()) {
                return Response.ok(alma.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Almacenamiento no encontrado con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba el almacenamiento.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear un nuevo almacenamiento",
	        description = "Crea un nuevo almacenamiento y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Almacenamiento creado exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Almacenamiento.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearAlmacenamiento(Almacenamiento alma) {
        try {
            Response.Status status = almaDAO.save(alma);
            return Response.status(status).entity(alma).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba el almacenamiento.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar un almacenamiento existente",
	        description = "Actualiza los detalles de un almacenamiento existente"
	       )
	@ApiResponse(responseCode = "200", description = "Almacenamiento actualizado exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Almacenamiento.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarAlmacenamiento(Almacenamiento alma) {
        try {
            Response.Status status = almaDAO.update(alma);
            return Response.status(status).entity(alma).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba el almacenamiento.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar un almacenamiento",
	        description = "Elimina un almacenamiento existente"
	       )
	@ApiResponse(responseCode = "204", description = "Almacenamiento eliminado exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarAlmacenamiento(Almacenamiento alma) {
		try {
	        Response.Status status = almaDAO.delete(alma);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba el almacenamiento.").build();
	    }
	}
	
}
