package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.Bien;
import persistance.interfaces.Iabm;

@Path("/Bien")
@Tag(name = "Bien", description = "Operaciones relacionadas con bienes")
public class BienResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Bien> bienDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todos los bienes",
    description = "Retorna una lista de todos los bienes disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de bienes obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Bien.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getBienes() {
        try {
            List<Bien> canales = bienDAO.getAll();
            return Response.ok(canales).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban los bienes.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener bien por ID",
	        description = "Devuelve un bien específico por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Bien encontrado",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Bien.class)))
	@ApiResponse(responseCode = "404", description = "Bien no encontrado")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getBienById(@PathParam("id") Long id) {
        try {
            Optional<Bien> canal = bienDAO.get(id);
            if (canal.isPresent()) {
                return Response.ok(canal.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Bien no encontrado con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba el Bien.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear un nuevo bien",
	        description = "Crea un nuevo bien y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Bien creado exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Bien.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearBien(Bien bien) {
        try {
            Response.Status status = bienDAO.save(bien);
            return Response.status(status).entity(bien).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba el Bien.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar un bien existente",
	        description = "Actualiza los detalles de un bien existente"
	       )
	@ApiResponse(responseCode = "200", description = "Bien actualizado exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Bien.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarBien(Bien bien) {
        try {
            Response.Status status = bienDAO.update(bien);
            return Response.status(status).entity(bien).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba el Bien.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar un bien",
	        description = "Elimina un bien existente"
	       )
	@ApiResponse(responseCode = "204", description = "Bien eliminado exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarBien(Bien bien) {
		try {
	        Response.Status status = bienDAO.delete(bien);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba el Bien.").build();
	    }
	}
	
}
