package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;

import javax.ws.rs.core.MediaType;

import models.Canal;
import persistance.interfaces.Iabm;

@Path("/Canal")
@Tag(name = "Canal", description = "Operaciones relacionadas con canales")
public class CanalResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Canal> canalDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todos los canales",
    description = "Retorna una lista de todos los canales disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de canales obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Canal.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getCanales() {
        try {
            List<Canal> canales = canalDAO.getAll();
            return Response.ok(canales).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban Canales.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener canal por ID",
	        description = "Devuelve un canal específico por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Canal encontrado",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Canal.class)))
	@ApiResponse(responseCode = "404", description = "Canal no encontrado")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getCanalById(@PathParam("id") Long id) {
        try {
            Optional<Canal> canal = canalDAO.get(id);
            if (canal.isPresent()) {
                return Response.ok(canal.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Canal no encontrado con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba el Canal.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear un nuevo canal",
	        description = "Crea un nuevo canal y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Canal creado exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Canal.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearCanal(Canal canal) {
        try {
            Response.Status status = canalDAO.save(canal);
            return Response.status(status).entity(canal).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba el Canal.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar un canal existente",
	        description = "Actualiza los detalles de un canal existente"
	       )
	@ApiResponse(responseCode = "200", description = "Canal actualizado exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Canal.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarCanal(Canal canal) {
        try {
            Response.Status status = canalDAO.update(canal);
            return Response.status(status).entity(canal).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba el Canal.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar un canal",
	        description = "Elimina un canal existente"
	       )
	@ApiResponse(responseCode = "204", description = "Canal eliminado exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarCanal(Canal canal) {
		try {
	        Response.Status status = canalDAO.delete(canal);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba el Canal.").build();
	    }
	}
	
}
