package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.FamiliaProductora;
import persistance.interfaces.Iabm;

@Path("/FamiliaProductora")
@Tag(name = "Familia Productora", description = "Operaciones relacionadas con familias productoras")
public class FamiliaProductoraResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<FamiliaProductora> FamProDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todas las familias productoras",
    description = "Retorna una lista de todas las familias prodcutoras disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de familias productora obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = FamiliaProductora.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getFamiliasProductoras() {
        try {
            List<FamiliaProductora> fams = FamProDAO.getAll();
            return Response.ok(fams).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban las familias productoras.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener familia productora por ID",
	        description = "Devuelve una familia productora específica por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Familia productora encontrada",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = FamiliaProductora.class)))
	@ApiResponse(responseCode = "404", description = "Familia prodcutora no encontrada")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getFamiliaProductoraById(@PathParam("id") Long id) {
        try {
            Optional<FamiliaProductora> fampro = FamProDAO.get(id);
            if (fampro.isPresent()) {
                return Response.ok(fampro.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Familia productora no encontrado con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba la familia productora.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear una nueva familia productora",
	        description = "Crea una nueva familia productora y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Familia productora creada exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = FamiliaProductora.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearFamiliaProductora(FamiliaProductora fam) {
        try {
            Response.Status status = FamProDAO.save(fam);
            return Response.status(status).entity(fam).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba la familia productora.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar una familia productora existente",
	        description = "Actualiza los detalles de una familia productora existente"
	       )
	@ApiResponse(responseCode = "200", description = "Familia productora actualizado exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = FamiliaProductora.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarFamiliaProductora(FamiliaProductora fam) {
        try {
            Response.Status status = FamProDAO.update(fam);
            return Response.status(status).entity(fam).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba la familia productora.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar una familia productora",
	        description = "Elimina una familia productora existente (de la base de datos, no manda un sicario xd)"
	       )
	@ApiResponse(responseCode = "204", description = "Familia productora eliminada exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarFamiliaProductora(FamiliaProductora fam) {
		try {
	        Response.Status status = FamProDAO.delete(fam);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba la familia productora").build();
	    }
	}
	
}
