package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.Nota;
import persistance.interfaces.Iabm;

@Path("/Nota")
@Tag(name = "Nota", description = "Operaciones relacionadas con la nota")
public class NotaResources {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Nota> notaDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todas las notas",
    description = "Retorna una lista de todas las notas disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de notas obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Nota.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getNotas() {
        try {
            List<Nota> notas = notaDAO.getAll();
            return Response.ok(notas).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban las notas.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener nota por ID",
	        description = "Devuelve una nota específica por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Nota encontrada",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Nota.class)))
	@ApiResponse(responseCode = "404", description = "Nota no encontrada")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getNotaById(@PathParam("id") Long id) {
        try {
            Optional<Nota> nota = notaDAO.get(id);
            if (nota.isPresent()) {
                return Response.ok(nota.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Nota no encontrada con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba la nota.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear una nueva nota",
	        description = "Crea una nueva nota y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Nota creada exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Nota.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearNota(Nota nota) {
        try {
            Response.Status status = notaDAO.save(nota);
            return Response.status(status).entity(nota).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba la nota.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar una nota existente",
	        description = "Actualiza los detalles de una nota existente"
	       )
	@ApiResponse(responseCode = "200", description = "Nota actualizada exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Nota.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarNota(Nota nota) {
        try {
            Response.Status status = notaDAO.update(nota);
            return Response.status(status).entity(nota).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba la nota.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar una nota",
	        description = "Elimina una nota existente"
	       )
	@ApiResponse(responseCode = "204", description = "Nota eliminada exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarNota(Nota nota) {
		try {
	        Response.Status status = notaDAO.delete(nota);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba la nota.").build();
	    }
	}
	
}
