package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.Orden;
import persistance.interfaces.Iabm;

@Path("/Orden")
@Tag(name = "Orden", description = "Operaciones relacionadas con ordenes")
public class OrdenResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Orden> ordenDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todas las ordenes",
    description = "Retorna una lista de todas las ordenes disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de ordenes obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Orden.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getOrdenes() {
        try {
            List<Orden> ordenes = ordenDAO.getAll();
            return Response.ok(ordenes).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban las ordenes.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener orden por ID",
	        description = "Devuelve una orden específica por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Orden encontrada",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Orden.class)))
	@ApiResponse(responseCode = "404", description = "Orden no encontrada")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getOrdenById(@PathParam("id") Long id) {
        try {
            Optional<Orden> orden = ordenDAO.get(id);
            if (orden.isPresent()) {
                return Response.ok(orden.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Orden no encontrada con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba la orden.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear una nueva orden",
	        description = "Crea una nueva orden y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Orden creada exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Orden.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearOrden(Orden orden) {
        try {
            Response.Status status = ordenDAO.save(orden);
            return Response.status(status).entity(orden).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba la orden.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar una orden existente",
	        description = "Actualiza los detalles de una orden existente"
	       )
	@ApiResponse(responseCode = "200", description = "Orden actualizada exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Orden.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarOrden(Orden orden) {
        try {
            Response.Status status = ordenDAO.update(orden);
            return Response.status(status).entity(orden).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba la orden.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar una orden",
	        description = "Elimina una orden existente"
	       )
	@ApiResponse(responseCode = "204", description = "Orden eliminada exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarOrden(Orden orden) {
		try {
	        Response.Status status = ordenDAO.delete(orden);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba la orden.").build();
	    }
	}
	
}
