package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.Receta;
import persistance.interfaces.Iabm;

@Path("/Receta")
@Tag(name = "Receta", description = "Operaciones relacionadas con recetas")
public class RecetaResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Receta> recDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todas las recetas",
    description = "Retorna una lista de todas las recetas disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de recetas obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Receta.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getRecetas() {
        try {
            List<Receta> recetas = recDAO.getAll();
            return Response.ok(recetas).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban las recetas.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener receta por ID",
	        description = "Devuelve una receta específica por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Receta encontrada",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Receta.class)))
	@ApiResponse(responseCode = "404", description = "Receta no encontrada")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getRecetaById(@PathParam("id") Long id) {
        try {
            Optional<Receta> receta = recDAO.get(id);
            if (receta.isPresent()) {
                return Response.ok(receta.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Receta no encontrada con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba la receta.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear una nueva receta",
	        description = "Crea una nueva receta y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Receta creada exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Receta.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearReceta(Receta receta) {
        try {
            Response.Status status = recDAO.save(receta);
            return Response.status(status).entity(receta).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba la receta.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar una receta existente",
	        description = "Actualiza los detalles de una receta existente"
	       )
	@ApiResponse(responseCode = "200", description = "Receta actualizada exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Receta.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarReceta(Receta receta) {
        try {
            Response.Status status = recDAO.update(receta);
            return Response.status(status).entity(receta).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba la receta.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar una receta",
	        description = "Elimina una receta existente"
	       )
	@ApiResponse(responseCode = "204", description = "Receta eliminada exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarReceta(Receta receta) {
		try {
	        Response.Status status = recDAO.delete(receta);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba la receta.").build();
	    }
	}

}
