package resourcesRest;

import java.util.List;
import java.util.Optional;

import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import models.Usuario;
import persistance.interfaces.Iabm;

@Path("/Usuario")
@Tag(name = "Usuario", description = "Operaciones relacionadas con usuario")
public class UsuarioResource {
	@Context
	UriInfo uriInfo;
	
	@Context
	Request request;
	
	@Inject
	private Iabm<Usuario> usuarioDAO;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(summary = "Obtener todos los usuarios",
    description = "Retorna una lista de todos los usuario disponibles")
	@ApiResponse(responseCode = "200", description = "Lista de usuarios obtenida exitosamente",
	        content = @Content(mediaType = "application/json", 
	        schema = @Schema(implementation = Usuario.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getUsuarios() {
        try {
            List<Usuario> usuarios = usuarioDAO.getAll();
            return Response.ok(usuarios).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaban Usuarios.").build();
        }
    }

	@GET
	@Path("{id}")
	@Produces(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Obtener usuario por ID",
	        description = "Devuelve un usuario específico por su ID"
	        )
	@ApiResponse(responseCode = "200", description = "Usuario encontrado",
		    content = @Content(mediaType = "application/json", 
		    schema = @Schema(implementation = Usuario.class)))
	@ApiResponse(responseCode = "404", description = "Usuario no encontrado")
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response getUsuarioById(@PathParam("id") Long id) {
        try {
            Optional<Usuario> usuario = usuarioDAO.get(id);
            if (usuario.isPresent()) {
                return Response.ok(usuario.get()).build();
            } else {
                return Response.status(Response.Status.NOT_FOUND).entity("Usuario no encontrado con ID: " + id).build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se buscaba el Usuario.").build();
        }
    }
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Crear un nuevo usuario",
	        description = "Crea un nuevo usuario y devuelve su id"
	     )
	@ApiResponse(responseCode = "201", description = "Usuario creado exitosamente",
	     content = @Content(mediaType = "application/json", 
	     schema = @Schema(implementation = Usuario.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
    public Response crearUsuario(Usuario usuario) {
        try {
            Response.Status status = usuarioDAO.save(usuario);
            return Response.status(status).entity(usuario).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se creaba el Usuario.").build();
        }
    }
	
	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Actualizar un usuario existente",
	        description = "Actualiza los detalles de un usuario existente"
	       )
	@ApiResponse(responseCode = "200", description = "Usuario actualizado exitosamente",
	    content = @Content(mediaType = "application/json", 
	    schema = @Schema(implementation = Usuario.class)))
	@ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response editarUsuario(Usuario usuario) {
        try {
            Response.Status status = usuarioDAO.update(usuario);
            return Response.status(status).entity(usuario).build();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se actualizaba el Usuario.").build();
        }
    }
	
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Operation(
	        summary = "Eliminar un usuario",
	        description = "Elimina un usuario existente"
	       )
	@ApiResponse(responseCode = "204", description = "Usuario eliminado exitosamente")
    @ApiResponse(responseCode = "500", description = "Error interno del servidor")
	public Response borrarUsuario(Usuario usuario) {
		try {
	        Response.Status status = usuarioDAO.delete(usuario);
	        return Response.status(status).build();
	    } catch (Exception e) {
	        e.printStackTrace();
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity("Un error surgio mientras se eliminaba el Usuario.").build();
	    }
	}
	

}
